import pytest
from unittest.mock import MagicMock, patch
from azure.core.exceptions import ResourceExistsError, ResourceNotFoundError
from aleph_secrets_manager.src.azure_keyvault_adaptor import AzureSecretsManager
from aleph_secrets_manager.src.port import Secret


@pytest.fixture
def mock_client():
    return MagicMock()


@pytest.fixture
def secrets_manager(mock_client):
    return AzureSecretsManager(mock_client)


def test_write_secret_success(secrets_manager, mock_client):
    secret = Secret(key="test_key", value="test_value")
    secrets_manager.write_secret(secret)
    mock_client.set_secret.assert_called_with("test-key", "test_value")


def test_write_secret_recover_deleted(secrets_manager, mock_client):
    secret = Secret(key="test_key", value="test_value")
    mock_client.set_secret.side_effect = [ResourceExistsError('"code": "ObjectIsDeletedButRecoverable"'), None]
    mock_client.begin_recover_deleted_secret.return_value = MagicMock()
    with patch("aleph_secrets_manager.src.azure_keyvault_adaptor.sleep") as mock_sleep:
        secrets_manager.write_secret(secret)
        mock_client.begin_recover_deleted_secret.assert_called_with("test-key")
        mock_sleep.assert_called()
        assert mock_client.set_secret.call_count == 2


def test_write_secret_other_exception(secrets_manager, mock_client):
    secret = Secret(key="test_key", value="test_value")
    mock_client.set_secret.side_effect = Exception("fail")
    with pytest.raises(Exception):
        secrets_manager.write_secret(secret)


def test_read_secret_success(secrets_manager, mock_client):
    mock_secret = MagicMock()
    mock_secret.name = "test-key"
    mock_secret.value = "test_value"
    mock_client.get_secret.return_value = mock_secret
    result = secrets_manager.read_secret("test_key")
    assert result.key == "test_key"
    assert result.value == "test_value"


def test_read_secret_not_found(secrets_manager, mock_client):
    mock_client.get_secret.side_effect = ResourceNotFoundError("not found")
    with pytest.raises(ValueError):
        secrets_manager.read_secret("test_key")


def test_delete_secret_success(secrets_manager, mock_client):
    poller = MagicMock()
    mock_client.begin_delete_secret.return_value = poller
    poller.result.return_value = None
    secrets_manager.delete_secret("test_key")
    mock_client.begin_delete_secret.assert_called_with("test-key")
    poller.result.assert_called()


def test_list_all_secrets(secrets_manager, mock_client):
    secret_property_1 = MagicMock()
    secret_property_1.name = 'test-key'
    secret_property_2 = MagicMock()
    secret_property_2.name = 'another-key'
    mock_secret_props = [secret_property_1, secret_property_2]
    mock_client.list_properties_of_secrets.return_value = mock_secret_props
    result = secrets_manager.list_all_secrets()
    assert result == ["test_key", "another_key"]


def test_read_all_secrets(secrets_manager, mock_client):
    secrets_manager.list_all_secrets = MagicMock(return_value=["test_key"])  # patch method
    secrets_manager.read_secret = MagicMock(return_value=Secret(key="test_key", value="val"))
    result = secrets_manager.read_all_secrets()
    assert result == [Secret(key="test_key", value="val")]



import pytest
from unittest.mock import MagicMock, patch
from azure.core.exceptions import ResourceExistsError, ResourceNotFoundError
from aleph_secrets_manager.src.azure_keyvault_adaptor import (
    AzureClientFactory,
    AzureSecretsManager,
)
from aleph_secrets_manager.src.port import Secret


@pytest.fixture(scope="session")
def client():
    vault_name = "testvault"
    client = AzureClientFactory.create(vault_name)
    return client


@pytest.fixture(scope="session")
def secrets_manager(client):
    secrets_manager = AzureSecretsManager(client)

    clear_all_secrets(secrets_manager=secrets_manager)
    yield secrets_manager
    clear_all_secrets(secrets_manager=secrets_manager)


def clear_all_secrets(secrets_manager: AzureSecretsManager):
    keys = secrets_manager.list_all_secrets()
    for key in keys:
        secrets_manager.delete_secret(key)


def test_read_write_secret_success(secrets_manager):
    secret = Secret(key="test_key", value="test_value")
    secrets_manager.write_secret(secret)

    retrieved_secret = secrets_manager.read_secret(secret.key)
    assert secret.key == retrieved_secret.key
    assert secret.value == retrieved_secret.value


def test_write_secret_recover_deleted(secrets_manager):
    secret = Secret(key="test_key", value="test_value")
    secrets_manager.write_secret(secret)

    secrets_manager.delete_secret(secret.key)

    secrets_manager.write_secret(secret)

    retrieved_secret = secrets_manager.read_secret(secret.key)
    assert secret.key == retrieved_secret.key
    assert secret.value == retrieved_secret.value


def test_read_secret_not_found(secrets_manager):
    with pytest.raises(Exception):
        secrets_manager.read_secret("MISSING_SECRET")


def test_delete_secret_success(secrets_manager):
    secret = Secret(key="test_key", value="test_value")
    secrets_manager.write_secret(secret)

    secrets_manager.delete_secret(secret.key)

    with pytest.raises(Exception):
        secrets_manager.read_secret(secret.key)


def test_list_all_secrets(secrets_manager):
    secrets = [
        Secret(key="test_key_1", value="test_value_1"),
        Secret(key="test_key_2", value="test_value_2"),
    ]
    for secret in secrets:
        secrets_manager.write_secret(secret)

    retrieved_secrets = secrets_manager.list_all_secrets()
    assert retrieved_secrets == [s.key for s in secrets]


def test_read_all_secrets(secrets_manager):
    secrets = [
        Secret(key="test_key_1", value="test_value_1"),
        Secret(key="test_key_2", value="test_value_2"),
    ]
    for secret in secrets:
        secrets_manager.write_secret(secret)

    retrieved_secrets = secrets_manager.read_all_secrets()
    assert retrieved_secrets == secrets



from abc import ABC
from pathlib import Path
from pydantic import BaseModel
from dotenv import dotenv_values

import typing as T
import logging


class Secret(BaseModel):
    key: str
    value: str


class SecretsManagerPort(ABC):
    def write_secret(self, secret: Secret):
        """Upload a secret to secret vault/manager"""
        raise NotImplementedError
    
    def read_secret(self, key: str) -> Secret:
        """Download a secret from a secrets vault/manager"""
        raise NotImplementedError
    
    def delete_secret(self, key: str):
        """Delete a secret from secret vault/manager"""
        raise NotImplementedError
    
    def list_all_secrets(self) -> T.List[str]:
        """List all secret names/keys from a secrets vault/manager"""
        raise NotImplementedError
    
    def read_all_secrets(self) -> T.List[Secret]:
        """Download all secrets""" 
        raise NotImplementedError
    
    def download_all_to_env_file(self, env_path: Path):
        """Download all secrets from a secrets vault/manager to a `.env` file"""
        
        secrets = self.read_all_secrets()
        with open(env_path, "w") as f: 
            for secret in secrets: 
                f.write(f"{secret.key}={secret.value}\n")

        logging.info(f"✅ Downloaded {len(secrets)} secrets to {str(env_path)}!")


    def upload_from_env_file(self, env_path: Path):
        """Upload all secrets to secret vault/manager from env file"""

        kv_pairs = dotenv_values(env_path)
        for key, value in kv_pairs.items():
            self.write_secret(Secret(key=key, value=value)) 

        logging.info(f"✅ Uploaded secrets from {str(env_path)}!")
    


from argparse import ArgumentParser
from pathlib import Path
from aleph_secrets_manager.src.port import Secret
from aleph_secrets_manager.src.azure_keyvault_adaptor import (
    AzureSecretsManager,
    AzureClientFactory,
)
import logging


def main():
    parser = ArgumentParser()
    parser.add_argument(
        "-v",
        "--vault-name",
        help="Name of Azure Key Vault to read/write to.",
        required=True,
    )
    subparsers = parser.add_subparsers(dest="command")

    read_parser = subparsers.add_parser("read", help="Read mode")
    read_parser.add_argument(
        "-f", "--file-path", help="File path to .env", required=True
    )

    write_parser = subparsers.add_parser("write", help="Write mode")
    write_parser.add_argument(
        "-f", "--file-path", help="File path to .env", required=True
    )

    delete_parser = subparsers.add_parser("delete", help="Delete mode")
    delete_parser.add_argument(
        "-k",
        "--keys",
        action="append",
        help="Keys to take action on. ",
    )

    args = parser.parse_args()
    logging.debug(f"⚙ Arguments received: {args}")

    secrets_manager = AzureSecretsManager(AzureClientFactory.create(args.vault_name))
    if args.command == "read":
        secrets_manager.download_all_to_env_file(env_path=Path(args.file_path))
        print(f"✅ Downloaded secrets to {str(args.file_path)}")
    elif args.command == "write":
        secrets_manager.upload_from_env_file(env_path=Path(args.file_path))
        print(f"✅ Uploaded secrets from {str(args.file_path)} to {args.vault_name}")
    elif args.command == "delete":
        for deletion in args.keys:
            secrets_manager.delete_secret(deletion)
        print(f"✅ Deleted secrets {args.keys} from {args.vault_name}")


if __name__ == "__main__":
    main()



from aleph_secrets_manager.src.port import Secret, SecretsManagerPort

import requests
import logging
from nacl import public, encoding
from base64 import b64encode
from pydantic import BaseModel


class GitHubPublicKey(BaseModel): 
    id: str
    value: str


# Guide: https://gist.github.com/comdotlinux/9a53bb00767a16d6646464c4b8249094
# REST API Reference: https://docs.github.com/en/rest/actions?apiVersion=2022-11-28#secrets
class GitHubSecretsManager(SecretsManagerPort): 
    def __init__(self, org: str, repo: str, auth_token: str):
        self.base_url = "https://api.github.com"
        self.org = org
        self.repo = repo
        self.token = auth_token
        self.headers = {
            "Authorization": f"Bearer {self.token}", 
            "X-GitHub-Api-Version": "2022-11-28"
        }
        raw_public_key = self._get_public_key(
            base_url=self.base_url, 
            org=self.org, 
            repo=self.repo, 
            auth_token=self.token
        )
        self.public_key = GitHubPublicKey(id=raw_public_key['key_id'], value=raw_public_key['key'])
        

    def _encrypt(self, public_key: str, unecrypted_value: str) -> str: 
        """Encrypt a secret value using the repo public key"""

        encoding_system = "utf-8"
        public_key_obj = public.PublicKey(public_key.encode(encoding_system), encoder=encoding.Base64Encoder())
        sealed_box = public.SealedBox(public_key_obj)
        encrypted_value = sealed_box.encrypt(unecrypted_value.encode(encoding_system))
        
        return b64encode(encrypted_value).decode(encoding_system)

    def _get_public_key(self, base_url: str, org: str, repo: str, auth_token: str):
        url = f"{base_url}/repos/{org}/{repo}/actions/secrets/public-key"
        headers = {"Authorization": f"Bearer {auth_token}"}
        
        response = requests.get(url=url, headers=headers, timeout=60)
        if response.status_code != 200: 
            logging.error(f"Response: {response.json()}")
            raise IOError(f"❌ Failed to get public key for {org} ({response.status_code}): \n{response.json()}")

        public_key_json = response.json()
        return public_key_json
    
    def list_all_secrets(self):
        """Retrieve all environment and repo secrets"""
        secrets = []
        url = f"{self.base_url}/repos/{self.org}/{self.repo}/actions/secrets"
        
        response = requests.get(url=url, headers=self.headers, timeout=60)
        if not response.ok: 
            logging.error(f"Response: {response.json()}")
            raise IOError(f"❌ Failed to list secrets for {self.org} ({response.status_code}): \n{response.json()}")

        secrets += response.json()["secrets"]
        return [s['name'] for s in secrets]
    
    def write_secret(self, secret: Secret):
        """Only repository secrets should be written"""

        encrypted_value: str = self._encrypt(self.public_key.value, secret.value)
        url = f"{self.base_url}/repos/{self.org}/{self.repo}/actions/secrets/{secret.key}"
        data = {
            "encrypted_value": encrypted_value,
            "key_id": self.public_key.id
        }

        response = requests.put(url=url, headers=self.headers, timeout=60, json=data)
        if not response.ok: 
            logging.error(f"Response: {response.json()}")
            raise IOError(f"❌ Failed to write secret {secret.key} ({response.status_code}): \n{response.json()}")

    def read_secret(self, key):
        """GitHub does not allow reading of secrets"""

        url = f"{self.base_url}/repos/{self.org}/{self.repo}/actions/secrets/{key}"
        response = requests.get(url=url, headers=self.headers, timeout=60)
        if not response.ok: 
            logging.error(f"Response: {response.json()}")
            raise IOError(f"❌ Failed to read secret {key} ({response.status_code}): \n{response.json()}")
        
        return Secret(key=response.json()['name'], value="")
        
    def delete_secret(self, key):
        url = f"{self.base_url}/repos/{self.org}/{self.repo}/actions/secrets/{key}"
        response = requests.delete(url=url, headers=self.headers, timeout=60)
        if not response.ok: 
            logging.error(f"Response: {response.json()}")
            if response.status_code == 404: 
                raise ValueError(f"❌ Failed to delete secret {key}: Does not exist")
            raise IOError(f"❌ Failed to delete secret {key} ({response.status_code}): \n{response.json()}")
        
    def read_all_secrets(self):
        secret_strings = self.list_all_secrets()
        return [self.read_secret(key) for key in secret_strings]


from argparse import ArgumentParser
from aleph_secrets_manager.src.gh_secrets_adaptor import GitHubSecretsManager
import logging
import os


def main():
    parser = ArgumentParser()
    parser.add_argument("-o", "--org", help="GitHub organization name", required=True)
    parser.add_argument("-r", "--repo", help="GitHub repository name", required=True)
    subparsers = parser.add_subparsers(dest="command")

    # List all secrets
    subparsers.add_parser("list", help="List all secret names in the repository")

    # Write a secret
    write_parser = subparsers.add_parser("write", help="Write a secret to the repository")
    write_parser.add_argument("-f", "--file-path", help="File path to .env", required=True)

    # Delete a secret
    delete_parser = subparsers.add_parser("delete", help="Delete a secret from the repository")
    delete_parser.add_argument("-k", "--key", required=True, help="Secret key name to delete")

    args = parser.parse_args()
    logging.debug(f"⚙ Arguments received: {args}")

    token = os.getenv('GH_AUTH_TOKEN')
    if not token:
        raise Exception("❌ Token has not been set. Please set the environment variable `GH_AUTH_TOKEN` with your personal access token.")
    secrets_manager = GitHubSecretsManager(org=args.org, repo=args.repo, auth_token=token)

    if args.command == "list":
        secrets = secrets_manager.list_all_secrets()
        print(f"Repository secrets: {secrets}")
    elif args.command == "write":
        secrets_manager.upload_from_env_file(env_path=args.file_path)
        print(f"✅ Uploaded secrets from {str(args.file_path)} to {args.org}/{args.repo}")
    elif args.command == "delete":
        secrets_manager.delete_secret(args.key)
        print(f"✅ Deleted secret {args.key} from {args.org}/{args.repo}")
    else:
        parser.print_help()

if __name__ == "__main__":
    main()



from time import sleep
from azure.keyvault.secrets import SecretClient
from azure.identity import DefaultAzureCredential
from azure.core.exceptions import ResourceExistsError, ClientAuthenticationError, ResourceNotFoundError
import re
import logging

from aleph_secrets_manager.src.port import Secret, SecretsManagerPort


class AzureClientFactory:
    @staticmethod
    def create(vault_name: str) -> SecretClient:
        credential = DefaultAzureCredential()
        vault_url = f"https://{vault_name}.vault.azure.net"
        client = SecretClient(vault_url=vault_url, credential=credential)
        return client


class AzureSecretsManager(SecretsManagerPort):
    def __init__(self, client: SecretClient):
        self.client = client

    def write_secret(self, secret):
        converted_key = self._convert_write(secret.key)
        successfully_deleted = False

        while not successfully_deleted: 
            try: 
                self.client.set_secret(converted_key, secret.value)
                successfully_deleted = True
            except ResourceExistsError as e:
                code = self._extract_error_code(e.message)

                if self._is_secret_recoverable(code): 
                    self.client.begin_recover_deleted_secret(converted_key) 
                    
                logging.warning(f"🚨 Encountered error writing {secret.key}: \n{e}")
                logging.warning(f"Retrying writing of {secret.key}...")
                sleep(1) 
            except Exception as e:
                raise

    def _extract_error_code(self, error_msg: str) -> str:
        """Get the internal error code for an Azure Resource error"""
        match = re.search(r'"code":\s*"([^"]+)"', error_msg)
        code = match.group(1)
        return code
    
    def _is_secret_recoverable(self, code: str) -> bool: 
        """Checks if a secret key can be recovered from an error code"""
        return code == "ObjectIsDeletedButRecoverable"

    def read_secret(self, key):
        converted_key = self._convert_write(key)

        try:
            az_secret = self.client.get_secret(converted_key)
            return Secret(key=self._convert_read(az_secret.name), value=az_secret.value)
        except ResourceNotFoundError:
            raise ValueError(f"❌ Failed to find secret '{key}' in {self.client.vault_url}")
        except Exception as e:
            raise
    
    def delete_secret(self, key):
        converted_key = self._convert_write(key)

        try:
            deleted_secret = self.client.begin_delete_secret(converted_key).result()
        except ResourceNotFoundError: 
            raise ValueError(f"❌ Failed to delete secret '{key}' from {self.client.vault_url} as it does not exist.")
        except Exception as e:
            raise
    
    def list_all_secrets(self):
        return [
            self._convert_read(secret_properties.name) 
            for secret_properties in self.client.list_properties_of_secrets()
        ]
    
    def read_all_secrets(self):
        keys = self.list_all_secrets()
        return [self.read_secret(key) for key in keys]
    
    def _convert_read(self, key: str): 
        return key.replace('-', '_')
    
    def _convert_write(self, key: str): 
        return key.replace('_', '-')